//
//  tank
//
//  Import and Playlist Daemon for autoradio project
//
//
//  Copyright (C) 2017-2018 Christian Pointner <equinox@helsinki.at>
//
//  This file is part of tank.
//
//  tank is free software: you can redistribute it and/or modify
//  it under the terms of the GNU General Public License as published by
//  the Free Software Foundation, either version 3 of the License, or
//  any later version.
//
//  tank is distributed in the hope that it will be useful,
//  but WITHOUT ANY WARRANTY; without even the implied warranty of
//  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
//  GNU General Public License for more details.
//
//  You should have received a copy of the GNU General Public License
//  along with tank. If not, see <http://www.gnu.org/licenses/>.
//

package store

import (
	//	"encoding/binary"
	"errors"
	//"strconv"
	"time"
)

var (
	ErrNotImplented = errors.New("not implemented")
	ErrNotFound     = errors.New("not found")
	ErrFileInUse    = errors.New("file is in use by at least one playlist")

//	InvalidID = uint64(0)
)

type TimeAndUser struct {
	User      string    `json:"user"`
	Timestamp time.Time `json:"timestamp"`
}

type ImportState int

const (
	ImportNew ImportState = iota
	ImportPending
	ImportRunning
	ImportDone
	ImportAborted
)

func (s ImportState) String() string {
	switch s {
	case ImportNew:
		return "new"
	case ImportPending:
		return "pending"
	case ImportRunning:
		return "running"
	case ImportDone:
		return "done"
	case ImportAborted:
		return "aborted"
	}
	return "unknown"
}

func (s *ImportState) fromString(str string) error {
	switch str {
	case "new":
		*s = ImportNew
	case "pending":
		*s = ImportPending
	case "running":
		*s = ImportRunning
	case "done":
		*s = ImportDone
	case "aborted":
		*s = ImportAborted
	default:
		return errors.New("invalid import state: '" + str + "'")
	}
	return nil
}

func (s ImportState) MarshalText() (data []byte, err error) {
	data = []byte(s.String())
	return
}

func (s *ImportState) UnmarshalText(data []byte) (err error) {
	return s.fromString(string(data))
}

type ImportLogLine struct {
	Timestamp time.Time `json:"timestamp"`
	Message   string    `json:"message"`
}

type ImportLog []ImportLogLine

func (l ImportLog) Append(message string) {
	l = append(l, ImportLogLine{time.Now(), message})
}

type Import struct {
	State   ImportState `json:"state"`
	Success bool        `json:"success"`
	Log     ImportLog   `json:"log"`
}

// stored in filesBn
type FileSource struct {
	FileName string `json:"filename"`
	Hash     string `json:"hash"`
	Import   Import `json:"import"`
}

type FileMetadata struct {
	Artist string `json:"artist"`
	Title  string `json:"title"`
	Album  string `json:"album"`
	// TODO: add more fields
}

type File struct {
	Created     TimeAndUser  `json:"created"`
	LastChanged TimeAndUser  `json:"last-changed"`
	Source      FileSource   `json:"source"`
	Metadata    FileMetadata `json:"metadata"`
	Size        uint64       `json:"size"`
	UsedBy      []uint64     `json:"used-by"`
}

func NewFile(user string) (f *File) {
	f = &File{}
	f.Created.User = user
	f.Created.Timestamp = time.Now()
	f.Source.Import.State = ImportNew
	return
}

func (f *File) Changed(user string) {
	f.LastChanged.User = user
	f.LastChanged.Timestamp = time.Now()
}

type Files map[uint64]File

// stored in filesBn
type Playlist struct {
	Created     TimeAndUser `json:"created"`
	LastChanged TimeAndUser `json:"last-changed"`
	// TODO: playlist storage layout
}

func NewPlaylist(user string) (p *Playlist) {
	p = &Playlist{}
	p.Created.User = user
	p.Created.Timestamp = time.Now()
	return
}

func (p *Playlist) Changed(user string) {
	p.LastChanged.User = user
	p.LastChanged.Timestamp = time.Now()
}

type Playlists map[uint64]Playlist
